--[[
Needs:
Text.lua
--]]

-- Work in progress





Hint = {
};



local color = Turbine.UI.Color;
local align = Turbine.UI.ContentAlignment;
local font = Turbine.UI.Lotro.Font;

-- local object;
-- local mouseLeave = nil;

-- Defaults
local FONT = "Verdana14";
local FONT_COLOR = "DarkKhaki";
local TEXT_WIDTH = 200;
local OUTER_OFFSET_V = 8;
local INNER_OFFSET = 7;
-- Border images
local CORNER_WIDTH = 10;
local CORNER_HEIGHT = 10;
local BORDER_WIDTH = 7;
local BORDER_HEIGHT = 7;


-- Name or code. Example: Verdana14 or 1107296257
Hint.Font = FONT;
-- Name or code. Example: DarkKhaki or Turbine.UI.Color( 0.5, 0.5, 0.5 )
Hint.FontColor = FONT_COLOR;
-- Number or "Single".
-- Number = limit width, multiline.
-- "Single" = dynamic width, single line. !! No checks for width
Hint.TextWidth = TEXT_WIDTH;
-- Hint's border - hint's text
Hint.InnerOffset = INNER_OFFSET;
-- Object's border - hint's border
Hint.OuterOffset = OUTER_OFFSET_V;






-- Forward declarations
--
local carrier;
local maxWidth;
local maxHeight;





-- Positions hint
local function positionHint( carrier, obj )
   -- !! If too long, there is no additional check
   -- Example: if there is not enough space under object, it'll create it above object,
   -- but it doesn't check, if there is enough space above...
   -- It should be ok, as long as hints are small

   local SCREEN_WIDTH, SCREEN_HEIGHT = Turbine.UI.Display:GetSize();
   local objWidth, objHeight = obj:GetSize();
   local screenX, screenY = obj:PointToScreen( 0, objHeight );
   local outerOffsetH = ( maxWidth - objWidth ) / 2;
   local left = screenX - outerOffsetH;
   -- Hint is larger than object. Correct horizontal position
   if ( outerOffsetH > 0 ) then
      -- Correct hint's left position
      if ( left < 0 ) then
         left = screenX;
      end
      -- Correct hint's right position
      if ( left + maxWidth > SCREEN_WIDTH ) then
         left = screenX + objWidth - maxWidth;
      end
   end
   local top = screenY + OUTER_OFFSET_V;
   -- Correct hint's vertical position
   if ( top + maxHeight > SCREEN_HEIGHT ) then
      top = screenY - objHeight - OUTER_OFFSET_V - maxHeight;
   end
   carrier:SetPosition( left, top );
end

-- Adds one part of the border's images
local function addHintBorderImage( parent, imageID, width, height )
   local obj = Turbine.UI.Control();
   obj:SetParent( parent );
   obj:SetBackground( imageID );
   obj:SetBlendMode( 4 ); -- Blend with background. Also works: 5, 7
   obj:SetSize( width, height );
   return obj;
end

-- Adds border's images
local function addHintBorder( parent )
   --[[ Фон и углы
   Ширина*высота

   Старые (до U12)
      Все - 19*19
      Левый верхний угол - 0x41000133
      Правый верхний угол - 0x41000139
      Правый нижний угол - 0x41000137
      Левый нижний угол - 0x41000135

      Верхняя граница - 0x4100013A; 2*3
      Нижняя граница - 0x41000136; 2*3
      Левая граница -  0x41000134; 3*2
      Правая граница - 0x41000138; 3*2

      Фон - 0x4100013B; 128*128

   Новые (U12)
      Все - 10*10
      Левый верхний угол - 0x4114061F
      Правый верхний угол - 0x41140618
      Правый нижний угол - 0x41140620
      Левый нижний угол - 0x41140619

      Верхняя граница - 0x4114061D; 10*7
      Нижняя граница - 0x4114061C; 10*7
      Левая граница -  0x4114061A; 7*10
      Правая граница - 0x4114061E; 7*10

      Фон - 0x4114061B; 283*382
   --]]

   -- Calculate size
   local parentW, parentH = parent:GetSize();
   local borderHeight = parentH - CORNER_HEIGHT * 2;
   local borderWidth = parentW - CORNER_WIDTH * 2;

   -- 1
   local topLeftCorner = addHintBorderImage( parent, 0x4114061F, CORNER_WIDTH, CORNER_HEIGHT );
   topLeftCorner:SetPosition( 0, 0 );
   -- 2
   local topBorder = addHintBorderImage( parent, 0x4114061D, borderWidth, BORDER_HEIGHT );
   topBorder:SetPosition( CORNER_WIDTH, 0 );
   -- 3
   local topRightCorner = addHintBorderImage( parent, 0x41140618, CORNER_WIDTH, CORNER_HEIGHT );
   topRightCorner:SetPosition( parentW - CORNER_WIDTH, 0 );
   -- 4
   local leftBorder = addHintBorderImage( parent, 0x4114061A, BORDER_WIDTH, borderHeight );
   leftBorder:SetPosition( 0, CORNER_HEIGHT );
   -- 5
   local rightBorder = addHintBorderImage( parent, 0x4114061E, BORDER_WIDTH, borderHeight );
   rightBorder:SetPosition( parentW - BORDER_WIDTH, CORNER_HEIGHT );
   -- 6
   local bottomLeftCorner = addHintBorderImage( parent, 0x41140619, CORNER_WIDTH, CORNER_HEIGHT );
   bottomLeftCorner:SetPosition( 0, parentH - CORNER_HEIGHT );
   -- 7
   local bottomBorder = addHintBorderImage( parent, 0x4114061C, borderWidth, BORDER_HEIGHT );
   bottomBorder:SetPosition( CORNER_WIDTH, parentH - BORDER_HEIGHT );
   -- 8
   local bottomRightCorner = addHintBorderImage( parent, 0x41140620, CORNER_WIDTH, CORNER_HEIGHT );
   bottomRightCorner:SetPosition( parentW - CORNER_WIDTH, parentH - CORNER_HEIGHT );
end


-- Creates and shows hint for an object.
-- If object doesn't have a MouseLeave event, adds one, that removes hint
Hint.show = function( obj, text )

   -- Check dependencies
   if ( not _Lib.Text ) then
      Turbine.Shell.WriteLine( ">> Hint. Error. Please, load Text.lua." );
      return;
   end

   -- Don't create if already have one
   if ( carrier ) then
      return;
   end


   local innerOffset = Hint.InnerOffset;
   local textWidth = Hint.TextWidth;


   -- Holds full construction
   carrier = Turbine.UI.Window();
   -- carrier:SetBackground( 0x4100013B );
   -- carrier:SetBackColor( Turbine.UI.Color.Black );
   carrier:SetVisible( true );
   carrier:SetZOrder( 100 );
   carrier:SetOpacity( 0 );

   local carrierBackground = Turbine.UI.Control();
   carrierBackground:SetParent( carrier );
   carrierBackground:SetBackground( 0x4114061B );
   carrierBackground:SetPosition( 2, 2 );

   -- Set and position text
   local c = color[Hint.FontColor] or Hint.FontColor;
   local f = font[Hint.Font] or Hint.Font;
   local label = Turbine.UI.Label();
   label:SetParent( carrier );
   label:SetZOrder( 1 ); -- To be above background images
   label:SetPosition( innerOffset, innerOffset );
   label:SetForeColor( c );
   label:SetTextAlignment( align.MiddleCenter );
   label:SetFont( f );
   label:SetText( text );


   -- Calculate and set size
   local textW, textH = Text:getSize( text, f, textWidth );
   maxWidth = textW + innerOffset * 2;
   maxHeight = textH + innerOffset * 2;
   carrier:SetSize( maxWidth, maxHeight );
   carrierBackground:SetSize( maxWidth - 2 * 2, maxHeight - 2 * 2 );
   label:SetSize( textW, textH );

   -- Add border, check position, show hint
   addHintBorder( carrier ); -- +9 KB. Problem: SetParent
   positionHint( carrier, obj );
   carrier:SetOpacity( 1 );

   -- If object doesn't have MouseLeave event, add hint's removal of a hint
   if ( not obj.MouseLeave ) then
      obj.MouseLeave = Hint.remove;
   else -- Rare small memory leak...
      -- object = obj;
      -- mouseLeave = obj.MouseLeave;
      -- obj.MouseLeave = { obj.MouseLeave, Hint.remove };
   end

end

-- Removes hint
Hint.remove = function()
   if ( carrier ) then
      carrier:GetControls():Clear();
      carrier = nil;
      -- if ( object ) then
         -- object.MouseLeave = mouseLeave;
      -- end
      -- object = nil;
      -- mouseLeave = nil;
   end
end





local mt = {};
mt.__call = function( _, ... )
   return Hint.show( ... );
end
setmetatable( Hint, mt );
